@@ -16,6 +16,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include "config.h"
+
 #include "manette-mapping-private.h"
 
 #include <errno.h>
@@ -113,8 +115,8 @@ bindings_array_has_destination_input (GArray *array,
 }
 
 static gboolean
-try_str_to_guint16 (gchar    *start,
-                    gchar   **end,
+try_str_to_guint16 (char     *start,
+                    char    **end,
                     guint16  *result)
 {
   manette_ensure_is_parseable (start);
@@ -128,8 +130,8 @@ try_str_to_guint16 (gchar    *start,
 }
 
 static gboolean
-parse_mapping_input_type (gchar                    *start,
-                          gchar                   **end,
+parse_mapping_input_type (char                     *start,
+                          char                    **end,
                           ManetteMappingInputType  *input_type)
 {
   manette_ensure_is_parseable (start);
@@ -158,16 +160,16 @@ parse_mapping_input_type (gchar                    *start,
 }
 
 static gboolean
-parse_mapping_index (gchar    *start,
-                     gchar   **end,
+parse_mapping_index (char     *start,
+                     char    **end,
                      guint16  *index)
 {
   return try_str_to_guint16 (start, end, index);
 }
 
 static gboolean
-parse_mapping_invert (gchar     *start,
-                      gchar    **end,
+parse_mapping_invert (char      *start,
+                      char     **end,
                       gboolean  *invert)
 {
   manette_ensure_is_parseable (start);
@@ -188,8 +190,8 @@ parse_mapping_invert (gchar     *start,
 }
 
 static gboolean
-parse_mapping_range (gchar                *start,
-                     gchar               **end,
+parse_mapping_range (char                 *start,
+                     char                **end,
                      ManetteMappingRange  *range)
 {
   manette_ensure_is_parseable (start);
@@ -216,8 +218,8 @@ parse_mapping_range (gchar                *start,
 }
 
 static gboolean
-parse_mapping_hat (gchar                *start,
-                   gchar               **end,
+parse_mapping_hat (char                 *start,
+                   char                **end,
                    guint16              *index,
                    ManetteMappingRange  *range,
                    gboolean             *invert)
@@ -265,15 +267,15 @@ parse_mapping_hat (gchar                *start,
 }
 
 static gboolean
-parse_destination_input (gchar    *start,
-                         gchar   **end,
+parse_destination_input (char     *start,
+                         char    **end,
                          guint16  *type,
                          guint16  *code)
 {
   const static struct {
     guint16 type;
     guint16 code;
-    const gchar *string_value;
+    const char *string_value;
   } values[] = {
     { EV_ABS, ABS_X, "leftx" },
     { EV_ABS, ABS_Y, "lefty" },
@@ -296,20 +298,20 @@ parse_destination_input (gchar    *start,
     { EV_KEY, BTN_TR2, "righttrigger" },
     { EV_KEY, BTN_Y, "x" },
     { EV_KEY, BTN_X, "y" },
-    { EV_KEY, BTN_TRIGGER_HAPPY7, "paddle1" },
-    { EV_KEY, BTN_TRIGGER_HAPPY8, "paddle2" },
-    { EV_KEY, BTN_TRIGGER_HAPPY9, "paddle3" },
-    { EV_KEY, BTN_TRIGGER_HAPPY10, "paddle4" },
+    { EV_KEY, BTN_TRIGGER_HAPPY2, "paddle1" },
+    { EV_KEY, BTN_TRIGGER_HAPPY1, "paddle2" },
+    { EV_KEY, BTN_TRIGGER_HAPPY4, "paddle3" },
+    { EV_KEY, BTN_TRIGGER_HAPPY3, "paddle4" },
+    { EV_KEY, BTN_TRIGGER_HAPPY5, "misc1" },
+    { EV_KEY, BTN_TRIGGER_HAPPY6, "misc2" },
+    { EV_KEY, BTN_TRIGGER_HAPPY7, "misc3" },
+    { EV_KEY, BTN_TRIGGER_HAPPY8, "misc4" },
+    { EV_KEY, BTN_TRIGGER_HAPPY9, "misc5" },
+    { EV_KEY, BTN_TRIGGER_HAPPY10, "misc6" },
     { EV_KEY, BTN_TRIGGER_HAPPY11, "touchpad" },
-    { EV_KEY, BTN_TRIGGER_HAPPY1, "misc1" },
-    { EV_KEY, BTN_TRIGGER_HAPPY2, "misc2" },
-    { EV_KEY, BTN_TRIGGER_HAPPY3, "misc3" },
-    { EV_KEY, BTN_TRIGGER_HAPPY4, "misc4" },
-    { EV_KEY, BTN_TRIGGER_HAPPY5, "misc5" },
-    { EV_KEY, BTN_TRIGGER_HAPPY6, "misc6" },
   };
-  const gint length = sizeof (values) / sizeof (values[0]);
-  gint i;
+  const int length = sizeof (values) / sizeof (values[0]);
+  int i;
 
   for (i = 0; i < length; i++)
     if (g_strcmp0 (start, values[i].string_value) == 0) {
@@ -354,7 +356,7 @@ append_binding (GArray                *type_array,
 }
 
 static gboolean
-parse_mapping_destination (gchar                 *destination,
+parse_mapping_destination (char                  *destination,
                            ManetteMappingBinding *binding)
 {
   if (!parse_mapping_range (destination,
@@ -375,7 +377,7 @@ parse_mapping_destination (gchar                 *destination,
 }
 
 static gboolean
-parse_mapping_source (gchar                 *source,
+parse_mapping_source (char                  *source,
                       ManetteMappingBinding *binding)
 {
   if (!parse_mapping_range (source,
@@ -436,7 +438,7 @@ parse_mapping_source (gchar                 *source,
 }
 
 static gboolean
-is_valid_guid (const gchar *string)
+is_valid_guid (const char *string)
 {
   if (!string)
     return FALSE;
@@ -453,14 +455,14 @@ is_valid_guid (const gchar *string)
  */
 static void
 set_from_mapping_string (ManetteMapping  *self,
-                         const gchar     *mapping_string,
+                         const char      *mapping_string,
                          GError         **error)
 {
   g_auto(GStrv) mappings = g_strsplit (mapping_string, ",", 0);
   guint mappings_length = g_strv_length (mappings);
   guint i = 0;
-  gchar *destination_string;
-  gchar *source_string;
+  char *destination_string;
+  char *source_string;
   ManetteMappingBinding binding = {};
 
   if (mappings_length < 2) {
@@ -541,7 +543,7 @@ g_array_try_free (GArray **array)
 }
 
 ManetteMapping *
-manette_mapping_new (const gchar  *mapping_string,
+manette_mapping_new (const char   *mapping_string,
                      GError      **error)
 {
   g_autoptr (ManetteMapping) self = NULL;
